<?php

include_once(dirname(__FILE__).'/Model.php');

class Module
{
  var $Id;
  var $Name;
  var $Version;
  var $License; 
  var $Creator;
  var $Description;
  var $Dependencies;
  var $Models = array();
  var $SupportedModels = array();
  var $Database;
  var $Installed;
  var $Initialized;
  var $System;
  
  function __construct($Database, $System)
  {
    $this->Database = &$Database;    
    $this->System = &$System;    
    $this->Initialized = false;
    $this->Installed = false;
  }
  
  function Install()
  {
    if($this->Installed) return;
    DebugLog('Installing module '.$this->Name.'...');
    $this->Installed = true;
    foreach($this->Dependencies as $Dependency)  
      $this->System->Modules[$Dependency]->Install();
    $this->LoadModels();
    foreach($this->Models as $Index => $Module)
    {
      $this->Models[$Index]->Install();
    }
    $this->Database->query('UPDATE SystemModule SET Installed=1 WHERE Name="'.$this->Name.'"');
  }
  
  function UnInstall()
  {
    DebugLog('Uninstalling module '.$this->Name.'...');
    $this->LoadModels();
    foreach(array_reverse($this->Models, true) as $Index => $Model)
    {
      $this->Models[$Index]->UnInstall();
    }
    $this->Database->query('UPDATE SystemModule SET Installed=0 WHERE Name="'.$this->Name.'"');
    $this->Installed = false;
  }
  
  function Init()
  {
    if($this->Initialized) return;
    $this->Initialized = true;
    foreach($this->Dependencies as $Dependency)  
      $this->System->Modules[$Dependency]->Init();
  } 
  
  function LoadModels()
  {
    $this->Models = array();
    foreach($this->SupportedModels as $ModelName)
    {
      $NewModel = new $ModelName($this->Database, $this->System);
      $NewModel->Module = &$this;
      $this->Models[$ModelName] = $NewModel;
    }       
  }
}

class ModularSystem
{
  var $Database;
  var $Modules = array();
  var $Models = array();
  var $Menu = array();
  
  function __construct($Database)
  {
    $this->Database = &$Database;
  }
  
  function ModulePresent($Name)
  {
    return(array_key_exists($Name, $this->Modules));
  }

  function LoadModules($Installed = true)
  {
    //DebugLog('Loading modules...');
    $this->Modules = array();
    $Query = 'SELECT `Id`, `Name`,`Installed` FROM `SystemModule`';
    if($Installed) $Query .= ' WHERE `Installed`=1';
      else $Query .= ''; // WHERE `Installed`=0';
    $DbResult = $this->Database->query($Query);
    while($Module = $DbResult->fetch_array())
    {
      //echo($Module['Name'].',');
      include_once('Modules/'.$Module['Name'].'/'.$Module['Name'].'.php');
      $ModuleClassName = 'Module'.$Module['Name'];
      $NewModule = new $ModuleClassName($this->Database, $this);      
      $NewModule->Id = $Module['Id'];
      $NewModule->Installed = $Module['Installed'];
      $this->Modules[$Module['Name']] = $NewModule;
    }     
  }
  
  function Init()
  {
    $this->LoadModules();
    foreach($this->Modules as $Index => $Module)
      $this->Modules[$Index]->Init();
  }
  
  function Install()
  {
    //DebugLog('Installing modular system core...');
    $this->Database->query('CREATE TABLE IF NOT EXISTS `SystemVersion` (
  `Id` int(11) NOT NULL AUTO_INCREMENT,
  `Version` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `Description` datetime NOT NULL,
  PRIMARY KEY (`Id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_czech_ci AUTO_INCREMENT=1;');
    $this->Database->query('CREATE TABLE IF NOT EXISTS `SystemModule` (
  `Id` int(11) NOT NULL AUTO_INCREMENT,
  `Name` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `Creator` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `Version` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `License` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `Installed` int(11) NOT NULL,
  `Description` text COLLATE utf8_czech_ci NOT NULL,
  PRIMARY KEY (`Id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_czech_ci AUTO_INCREMENT=1;');
    
    $this->Database->query('CREATE TABLE IF NOT EXISTS `SystemModuleDependency` (
  `Id` int(11) NOT NULL AUTO_INCREMENT,
  `Module` int(11) NOT NULL,
  `DependencyModule` int(11) NOT NULL,
  PRIMARY KEY (`Id`),
  KEY (`Module`),
  KEY (`DependencyModule`),
  UNIQUE (`Module` , `DependencyModule`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_czech_ci AUTO_INCREMENT=1;');
    $this->Database->query('ALTER TABLE `SystemModuleDependency` ADD CONSTRAINT `SystemModuleDependency_ibfk_1` FOREIGN KEY ( `Module` ) REFERENCES `SystemModule` (`Id`)');
    $this->Database->query('ALTER TABLE `SystemModuleDependency` ADD CONSTRAINT `SystemModuleDependency_ibfk_2` FOREIGN KEY ( `DependencyModule` ) REFERENCES `SystemModule` (`Id`)');
    
    $this->Database->query('CREATE TABLE IF NOT EXISTS `SystemModel` (
  `Id` int(11) NOT NULL AUTO_INCREMENT,
  `Name` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `Module` int(11) NOT NULL,
  KEY (`Module`),
  PRIMARY KEY (`Id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_czech_ci AUTO_INCREMENT=1;');
    $this->Database->query('ALTER TABLE `SystemModel` ADD CONSTRAINT `SystemModel_ibfk_1` FOREIGN KEY ( `Module` ) REFERENCES `SystemModule` (`Id`)');
    
    $this->Database->query('CREATE TABLE IF NOT EXISTS `SystemModelProperty` (
  `Id` int(11) NOT NULL AUTO_INCREMENT,
  `Name` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `Type` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `Model` int(11) NOT NULL,
  KEY (`Model`),
  PRIMARY KEY (`Id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_czech_ci AUTO_INCREMENT=1;');
    $this->Database->query('ALTER TABLE `SystemModelProperty` ADD CONSTRAINT `SystemModelProperty_ibfk_1` FOREIGN KEY ( `Model` ) REFERENCES `SystemModel` (`Id`)');
    
    $this->ReloadList();
    $this->LoadModules(false);
    //$this->Modules['System']->Install();
    foreach($this->Modules as $Index => $Module)
    {      
      $this->Modules[$Index]->Install();
    }
  }
  
  function UnInstall()
  {
    DebugLog('Uninstalling modular system core...');
    foreach(array_reverse($this->Modules, true) as $Index => $Module)
      $this->Modules[$Index]->UnInstall();
    
    // Delete tables with reverse order
    $this->Database->query('ALTER TABLE `SystemModelProperty` DROP FOREIGN KEY `SystemModelProperty_ibfk_1`');
    $this->Database->query('DROP TABLE IF EXISTS `SystemModelProperty`');
    $this->Database->query('ALTER TABLE `SystemModel` DROP FOREIGN KEY `SystemModel_ibfk_1`');
    $this->Database->query('DROP TABLE IF EXISTS `SystemModel`');
    $this->Database->query('ALTER TABLE `SystemModuleDependency` DROP FOREIGN KEY `SystemModuleDependency_ibfk_1`');
    $this->Database->query('ALTER TABLE `SystemModuleDependency` DROP FOREIGN KEY `SystemModuleDependency_ibfk_2`');
    $this->Database->query('DROP TABLE IF EXISTS `SystemModuleDependency`');
    $this->Database->query('DROP TABLE IF EXISTS `SystemModule`');
    $this->Database->query('DROP TABLE IF EXISTS `SystemVersion`');
  }
  
  function IsInstalled()
  {
    $DbResult = $this->Database->query('SELECT table_name FROM information_schema.tables
WHERE table_schema = "'.$this->Database->Database.'" AND table_name = "SystemVersion";');    
    if($DbResult->num_rows > 0) return(true); 
      else return(false);
  }
  
  function ReloadList()
  {
    // Load list of modules from database
    $Modules = array();
    $DbResult = $this->Database->query('SELECT * FROM `SystemModule`');
    while($DbRow = $DbResult->fetch_assoc())
      $Modules[$DbRow['Name']] = $DbRow;
    
    // Load list of modules on disk
    $ModulesOnDisk = array();
    $Files = scandir('Modules'); 
    foreach($Files as $File)
    if(is_dir('Modules/'.$File) and ($File != '.') and ($File != '..') and (substr($File, 0, 1) != '.'))
    {
      $ModulesOnDisk[] = $File;
    }   

    // Add new 
    foreach($ModulesOnDisk as $ModuleName)
    if(!array_key_exists($ModuleName, $Modules))
    {
      DebugLog('Adding module '.$ModuleName.' to list');
      include_once('Modules/'.$ModuleName.'/'.$ModuleName.'.php');
      $ModuleClassName = 'Module'.$ModuleName;
      if(class_exists($ModuleClassName))
      {
        $Module = new $ModuleClassName($this->Database, $this); 
        $this->Database->insert('SystemModule', array('Name' => $Module->Name, 
          'Version' => $Module->Version, 'Creator' => $Module->Creator, 
          'Description' => $Module->Description, 'License' => $Module->License,
          'Installed' => 0));
        unset($Module);
      } else throw new Exception('Missing class '.$ModuleClassName.' in module '.$ModuleName);
    }
    
    // Remove missing
    foreach($Modules as $Module)
    if(($Module['Installed'] == 0) and !in_array($Module['Name'], $ModulesOnDisk))
    {
      DebugLog('Removing module '.$Module['Name'].' from list');
      $this->Database->query('DELETE FROM `SystemModule` WHERE `Id` = '.$Module['Id']);
    }
    
    // Reload dependencies
    $this->LoadModules(false);
    foreach($this->Modules as $Module)
    {
      foreach($Module->Dependencies as $Dependency)
      {
        $this->Database->insert('SystemModuleDependency', array('Module' => $Module->Id, 
          'DependencyModule' => $this->Modules[$Dependency]->Id));
      }
    }
  }
}

?>